#pragma once

// Databases
#include "FileIO/InventoryParts.h"
#include "FileIO/GottenBricks.h"
#include "FileIO/Inventories.h"
#include "FileIO/InventorySets.h"
#include "FileIO/Sets.h"
#include "FileIO/Colors.h"
#include "FileIO/Parts.h"
#include "FileIO/Elements.h"
#include "FileIO/PartCategories.h"
#include "FileIO/PartRelationships.h"
#include "FileIO/Themes.h"

// UI
#include "UI/Button.h"
#include "UI/Text.h"
#include "UI/Tile.h"
#include "UI/DropDown.h"
#include "UI/EditableTextbox.h"
#include "UI/Image.h"

#include <string>
#include <SDL.h>
#include <vector>
#include <glm/vec2.hpp>
#include <glm/vec4.hpp>

#define MIN(a,b)		((a) < (b) ? (a) : (b))
#define MAX(a,b)		((a) > (b) ? (a) : (b))
#define CLAMP(x,a,b)	MAX(MIN((x), (b)), (a))

constexpr int SCRWIDTH =	1920;
constexpr int SCRHEIGHT =	1080;
constexpr int DISPLAY = 0;

constexpr int TEXTSIZE = 4;
constexpr int IMAGESIZE = 20;

// UI colors
constexpr glm::ivec4 BLACK = glm::ivec4(0, 0, 0, 0);
constexpr glm::ivec4 WHITE = glm::ivec4(255, 255, 255, 0);
constexpr glm::ivec4 DARKGREY = glm::ivec4(50, 50, 50, 0);
constexpr glm::ivec4 GREY = glm::ivec4(100, 100, 100, 0);
constexpr glm::ivec4 RED = glm::ivec4(255, 0, 0, 0);
constexpr glm::ivec4 DEBUGPINK = glm::ivec4(255, 0, 255, 0);

// Database
#ifdef _DEBUG
constexpr bool DEBUG = true;
#else
constexpr bool DEBUG = false;
#endif
constexpr bool LoadInventoryParts =		!DEBUG || true;
constexpr bool LoadInventories =		!DEBUG || true;
constexpr bool LoadInventorySets =		!DEBUG || true;
constexpr bool LoadSets =				!DEBUG || true;
constexpr bool LoadColors =				!DEBUG || true;
constexpr bool LoadParts =				!DEBUG || true;
constexpr bool LoadElements =			!DEBUG || true;
constexpr bool LoadPartCategories =		!DEBUG || true;
constexpr bool LoadPartRelationships =	!DEBUG || true;
constexpr bool LoadThemes =				!DEBUG || true;
constexpr bool LoadMinifigSets =		!DEBUG || true;
constexpr bool LoadGottenBricks =		!DEBUG || true;

constexpr bool SearchOnChangeEditableTextbox = !DEBUG;


enum struct Menus
{
	NONE,
	LoadImagesScreen,
	StartScreen,
	BrickSelection,
	SetSelection,
	ConfirmBrickSelection,
	BrickInfo,
	SetInfo,
};

class Manager
{
public:
	// Methods
	// Rendering
	int InitRenderer();
	void CheckEvents();
	void ClearScreen();
	void RenderScreen();
	void CapFPS(int FPS);

	// UI
	void UpdateUI();
	void RenderUI();
	void RemoveUI();
	void CreateLoadImagesScreen();
	void CreateStartScreen();
	void CreateBrickSelection();
	void CreateSetSelection();
	void CreateBrickInfo();
	void CreateSetInfo();
	void CreateConfirmBrickSelection();

	// File IO
	void OpenFiles();
	void OpenCustomFiles();
	void LoadImages();
	void GottenBricksToCSV();
	void GottenSetsToCSV();

	// Miscellaneous
	void CreateListOfParts();
	void CheckPossibleSets();



	// Variables
	// Rendering
	SDL_Window* m_Window = nullptr;
	SDL_Surface* m_WindowSurface = nullptr;
	SDL_Renderer* m_Renderer = nullptr;
	SDL_Event m_Event = SDL_Event();
	glm::ivec2 m_ScreenSize = glm::ivec2();
	const glm::ivec4 m_ClearColor = glm::ivec4(40, 40, 60, 0);
	bool m_ExitApp = false;
	bool m_ShouldRender = true;
	int m_FrameCount = 0;
	float m_DeltaTime = 0.0001f;
	
	// Input
	bool m_MouseButtonDown = false;
	glm::ivec2 m_MousePos = glm::ivec2();

	// Database values
	std::vector<InventoryParts> m_InventoryParts;
	std::vector<InventorySets> m_InventorySets;
	std::vector<Inventories> m_Inventories;
	std::vector<Sets> m_Sets;
	std::vector<Colors> m_Colors;
	std::vector<Parts> m_Parts;
	std::vector<PartCategories> m_PartCategories;
	std::vector<PartRelationships> m_PartRelationships;
	std::vector<Elements> m_Elements;
	std::vector<Themes> m_Themes;
	std::vector<std::string> m_MinifigSets;

	// Miscellaneous
	std::vector<GottenBricks> m_AllBricks;
	std::vector<Sets> m_PossibleSets;


	// UI Variables
	// General
	std::vector<UIParent*> m_UI;
	Menus m_ScreenToLoad = Menus::NONE;
	Menus m_CurrentMenu = Menus::NONE;
	float m_DisplayTextboxCursor = 0.f;
	bool m_EditingTextbox = false;
	bool m_ShowSelectedOnly = false;	// Should it only show the selected bricks, or all
	bool m_CustomFilesLoaded = false;

	// BrickSelection menu
	int m_BS_Page = 1;	// Start count from 1 for user
	int m_BS_BricksSelected = 0;	// How many bricks have a quantity > 0
	std::vector<GottenBricks> m_BS_DisplayBricks;
	bool m_BS_ShouldReloadBricks = true;
	int m_BS_Color = -2;	// -2 is used for show all, as -1 is already in use
	std::string m_BS_PartNum = "";

	// BrickInfo menu
	GottenBricks* m_BI_SelectedBrick = nullptr;

	// SetSelection menu
	int m_SS_Page = 1;	// Start count from 1 for user
	std::vector<Sets> m_SS_DisplaySets;
	bool m_SS_ShouldReloadSets = true;
	bool m_SS_ComingFromBS = false;
	std::string m_SS_SetNum = "";
	bool m_SS_ShowMinifigSets = false;

	// SetInfo menu
	Sets* m_SI_SelectedSet = nullptr;

private:
	std::vector<GottenBricks> GetDisplayableBricks(
		std::vector<GottenBricks>& input,
		bool CheckColor,
		bool CheckPartNum,
		bool CheckQuantity);

	std::vector<Sets> GetDisplayableSets(
		std::vector<Sets> input,
		bool CheckSetNum,
		bool CheckQuantity,
		bool CheckMinifigSets);
};